#!/usr/bin/perl

###### mapviewToBed.pl
## Author: APB
## May 20, 2008
##
## Convert maq mapview output to bed format.
##  (http://genome.ucsc.edu/FAQ/FAQformat#format1)
## Written for use by F-Seq.
##
## This will output all sequence reads that are < MAX_HITS with 0 or 1
##  mismatches.
##
######

#------------------------------------------------------------------------------
# Usage requires input directory as argument
#
$usage = qq(
Usage:  $0 <MAX_HITS> <OFFSET> <FILE>
\tMAX_HITS\t= Exclude alignments with >= this number of hits.
\tOFFSET\t\t= Fragment size of read origin (for 5' end use 1).
\tFILE\t\t= Mapview file to convert.
\n);

die($usage) if (@ARGV < 3);
#------------------------------------------------------------------------------

use strict;

my $MAX_HITS = $ARGV[0]; #non-inclusive
my $OFFSET = $ARGV[1];
my $FILE = $ARGV[2];

open(MAPV, $FILE) or die("Could not open file!");
while(my $line = <MAPV>) {
    chomp($line);
    my @data = split('\t', $line);
    if($data[9] == 0) {     #Best hit has 0 mismatches
        if($data[11] < $MAX_HITS) {
            print print_bed(\@data, $OFFSET);
        }
    } elsif($data[9] == 1) {    #Best hit has 1 mismatch
        if($data[12] < $MAX_HITS) {
            print print_bed(\@data, $OFFSET);
        }
    } else {        #best hit has > 1 mismatches
        #do nothing
    }
}
close(MAPV);
exit(0);

sub print_bed {
    my($arr_ptr, $offset) = shift;
    my @toprint = @$arr_ptr;

    my $start;
    my $end;
    if($strand eq "+") {
        $start = $toprint[2];
        $end = $toprint[2] + $offset;
    } elsif($strand eq "-") {
        $end = $toprint[2];
        $start = $toprint[2] - $offset;
    }

    my $return_str = "";
    $return_str .= $toprint[1]; #chrom
    $return_str .= "\t";
    $return_str .= $start; #start
    $return_str .= "\t";
    $return_str .= $end; #end
    $return_str .= "\t";
    $return_str .= $toprint[0]; #name
    $return_str .= "\t";
    $return_str .= $toprint[6]; #score
    $return_str .= "\t";
    $return_str .= $toprint[3]; #strand
    $return_str .= "\n";
    return $return_str;
}
